@@ -1,4 +1,4 @@
-from crocs.regex import Join, Include,Any
+from crocs.regex import Pattern, Include,Any
 
 e = Any(Include('a', 'b', 'c'), '1')
 
