@@ -1,6 +1,6 @@
-from crocs.regex import Seq, Include, Repeat, Join, NamedGroup, Include
+from crocs.regex import Seq, Include, Repeat, Pattern, NamedGroup, Include
 
-# First we define how our Joins look like.
+# First we define how our Patterns look like.
 name_valid_letters = Seq('a', 'z')
 name_valid_numbers = Seq('0', '9')
 name_valid_signs   = '_.-'
@@ -11,11 +11,11 @@ name_valid_chars = Include(name_valid_letters,
 name_valid_numbers, name_valid_signs)
 
 # Think of the Repeat class as meaning: fetch the
-# described Joins one or more Repeat.
+# described Patterns one or more Repeat.
 name_chunk = Repeat(name_valid_chars, 1)
 
 # The first letter in the mail name has to be a in 'a-z'.
-name_fmt = Join(Include(name_valid_letters), name_chunk)
+name_fmt = Pattern(Include(name_valid_letters), name_chunk)
 
 # Think of group as a way to keep reference
 # to the fetched chunk.
@@ -27,8 +27,8 @@ name = NamedGroup('name', name_fmt)
 hostname_chars = Include(name_valid_letters)
 hostname_chunk = Repeat(hostname_chars, 1)
 
-# We format finally the complete hostname Join.
-hostname_fmt = Join('python', hostname_chunk)
+# We format finally the complete hostname Pattern.
+hostname_fmt = Pattern('python', hostname_chunk)
 
 # Keep reference for the group.
 hostname = NamedGroup('hostname', hostname_fmt)
@@ -37,6 +37,6 @@ hostname = NamedGroup('hostname', hostname_fmt)
 domain  = NamedGroup('domain', 'br')
 
 # Finally we generate the regex and check how it looks like.
-match_mail = Join(name, '@', hostname, '.', domain)
+match_mail = Pattern(name, '@', hostname, '.', domain)
 match_mail.test()
 match_mail.hits()
